/*
 * This code is part of the strategy game operational-space.
 * operational-space comes with ABSOLUTELY NO WARRANTY and is licensed under GPL-2.0.
 *    Copyright (C) 2024  VegOwOtenks, Sleppo04
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
#include "utf-8.h"

#include <stddef.h>
#include <stdint.h>
#include <string.h>

const char SINGLE_BYTE_CODEPOINT = 0x00;
const char DOUBLE_BYTE_CODEPOINT = 0x06;
const char TRIPLE_BYTE_CODEPOINT = 0x0E;
const char QUADRUPLE_BYTE_CODEPOINT = 0x1E;

size_t UTF8_CodepointLength(const char* string)
{
    char character = *string;
    character >>= 7;
    if (memcmp(&character, &SINGLE_BYTE_CODEPOINT, 1) == 0) {
        return 1;
    }

    character = (*string) >> 5;
    if (memcmp(&character, &DOUBLE_BYTE_CODEPOINT, 1) == 0) {
        return 2;
    }

    character = (*string) >> 4;
    if (memcmp(&character, &TRIPLE_BYTE_CODEPOINT, 1) == 0) {
        return 3;
    }

    character = (*string) >> 3;
    if (memcmp(&character, &QUADRUPLE_BYTE_CODEPOINT, 1) == 0) {
        return 4;
    }

    return SIZE_MAX;
}

const char* UTF8_NextCodepoint(const char* string)
{
	return string + UTF8_CodepointLength(string);
}
